"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const type_analyzer_1 = require("@previewjs/type-analyzer");
const vfs_1 = require("@previewjs/vfs");
const path_1 = __importDefault(require("path"));
const vitest_1 = require("vitest");
const _1 = __importDefault(require("."));
const analyze_component_1 = require("./analyze-component");
const svelte_reader_1 = require("./svelte-reader");
const ROOT_DIR_PATH = path_1.default.join(__dirname, "virtual");
const MAIN_FILE = path_1.default.join(ROOT_DIR_PATH, "App.svelte");
(0, vitest_1.describe)("analyze Svelte component", () => {
    let memoryReader;
    let typeAnalyzer;
    let frameworkPlugin;
    (0, vitest_1.beforeEach)(async () => {
        memoryReader = (0, vfs_1.createMemoryReader)();
        frameworkPlugin = await _1.default.create();
        typeAnalyzer = (0, type_analyzer_1.createTypeAnalyzer)({
            rootDirPath: ROOT_DIR_PATH,
            reader: (0, svelte_reader_1.createSvelteTypeScriptReader)((0, vfs_1.createStackedReader)([
                memoryReader,
                (0, vfs_1.createFileSystemReader)({
                    watch: false,
                }),
                (0, vfs_1.createFileSystemReader)({
                    mapping: {
                        from: path_1.default.join(__dirname, "..", "preview", "modules"),
                        to: path_1.default.join(ROOT_DIR_PATH, "node_modules"),
                    },
                    watch: false,
                }),
            ])),
            tsCompilerOptions: frameworkPlugin.tsCompilerOptions,
        });
    });
    (0, vitest_1.afterEach)(() => {
        typeAnalyzer.dispose();
    });
    (0, vitest_1.test)("no props", async () => {
        (0, vitest_1.expect)(await analyze(`
<script lang="ts">
  const increment = () => {
    count += 1;
  };
</script>

<button on:click={increment}>
  Hello, World!
</button>
`)).toEqual({
            propsType: (0, type_analyzer_1.objectType)({}),
            types: {},
        });
    });
    (0, vitest_1.test)("props without type", async () => {
        (0, vitest_1.expect)(await analyze(`
<script lang="ts">
  export let count;
  const increment = () => {
    count += 1;
  };
</script>

<button on:click={increment}>
  {count}
</button>
`)).toEqual({
            propsType: (0, type_analyzer_1.objectType)({
                count: type_analyzer_1.ANY_TYPE,
            }),
            types: {},
        });
    });
    (0, vitest_1.test)("props without default", async () => {
        (0, vitest_1.expect)(await analyze(`
<script lang="ts">
  export let count: number;
  const increment = () => {
    count += 1;
  };
</script>

<button on:click={increment}>
  {count}
</button>
`)).toEqual({
            propsType: (0, type_analyzer_1.objectType)({
                count: type_analyzer_1.NUMBER_TYPE,
            }),
            types: {},
        });
    });
    (0, vitest_1.test)("props with default", async () => {
        (0, vitest_1.expect)(await analyze(`
<script lang="ts">
  export let count = 12;
  const increment = () => {
    count += 1;
  };
</script>

<button on:click={increment}>
  {count}
</button>
`)).toEqual({
            propsType: (0, type_analyzer_1.objectType)({
                count: (0, type_analyzer_1.optionalType)(type_analyzer_1.NUMBER_TYPE),
            }),
            types: {},
        });
    });
    (0, vitest_1.test)("readonly props", async () => {
        (0, vitest_1.expect)(await analyze(`
<script lang="ts">
  export const count = 12;
  const increment = () => {
    count += 1;
  };
</script>

<button on:click={increment}>
  {count}
</button>
`)).toEqual({
            propsType: (0, type_analyzer_1.objectType)({}),
            types: {},
        });
    });
    async function analyze(source) {
        memoryReader.updateFile(MAIN_FILE, source);
        return (0, analyze_component_1.analyzeSvelteComponentFromSFC)(typeAnalyzer, MAIN_FILE);
    }
});
